# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/nc-to-thredds/source/ReadMetaDataAdjusted_NoLog.py $
# Last committed:     $Revision: 686988 $; $Author: vvuik $; $Date: 2020-06-12 16:00:59 +0200 (vr, 12 jun 2020) $
#
# -*- coding: utf-8 -*-

"""
Andere versies van dit script staan op locaties:
    C:\dms\code\python-ftp-svn\source\mics\ReadMetaData.py
    C:\fews\clients\oc\Modules\Waterlevels\waterlevels\ReadMetaDataAdjusted.py
"""

import numpy as np
import pandas as pd
pd.options.mode.chained_assignment = None  # default='warn'
import re
from xlrd.biffh import XLRDError


class ReadMetaDataAdjusted:

    def __init__(self, fn, ts, request_location):
        self.fn = fn
        self.ts = ts
        self.current_meta_data = {}
        self.parse_file(request_location)

    def insert_tuple(self, k, t, v):
        if k not in self.current_meta_data:
            self.current_meta_data[k] = {t: v}
        else:
            tmpTup = self.current_meta_data[k]
            tmpTup[t] = v
            self.current_meta_data[k] = tmpTup

    def parse_file(self, request_location):
        print("ReadMetaDataAdjusted.parse_file: reading {}".format(self.fn))
        xls = pd.ExcelFile(self.fn)

        # map request location to the names used in the xls
        #if request_location == 'BUOY1':
        #    local_request_location = 'FMW'
        #else:
        local_request_location = request_location.replace('_', '')

        print("ReadMetaDataAdjusted.parse_file: searching for sheet {}".format(local_request_location))

        try:
            df = pd.read_excel(self.fn, sheet_name=local_request_location)
            df = df[df.Item.notna()]
            df['NAP Z'] = df['NAP Z'].map(lambda x: np.nan if isinstance(x, str) else x)
            # dataframe of the input parameter ts (timestamps)
            df_datum = pd.DataFrame(data={'Datum': self.ts})

            df_bordes1 = df[df.Item.str.match("^Bordes", flags=re.IGNORECASE)]
            if not df_bordes1.empty:
                df_bordes = pd.merge_asof(left=df_datum, right=df_bordes1[['Datum', 'RD X', 'RD Y']], on='Datum', direction='backward')
                df_bordes.fillna(df_bordes1.iloc[0][['RD X', 'RD Y']], inplace=True)
                x = df_bordes.iloc[0]['RD X']
                y = df_bordes.iloc[0]['RD Y']
                name = df_bordes1.iloc[0]['Locatie']
                self.insert_tuple(request_location, 'X', x)
                self.insert_tuple(request_location, 'Y', y)
                self.insert_tuple(request_location, 'Naam', name)
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('X', x))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('Y', y))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('Naam', name))

            df_bodem1 = df[df.Item.str.match("^Bodem", flags=re.IGNORECASE)]
            df_bodem1['metadataDatum'] = df_bodem1['Datum']
            if not df_bodem1.empty:
                df_bodem = pd.merge_asof(left=df_datum, right=df_bodem1[['Datum', 'NAP Z', 'metadataDatum']], on='Datum', direction='backward')
                df_bodem.fillna(df_bodem1.iloc[0][['NAP Z']], inplace=True)
                nap_z = df_bodem.iloc[0]['NAP Z']
                metadataDatum = df_bodem.iloc[0]['metadataDatum'].strftime("%Y-%m-%d")
                self.insert_tuple(request_location, 'bodem_NAP_Z', nap_z)
                self.insert_tuple(request_location, 'bodem_NAP_Z_date', metadataDatum)
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('bodem_NAP_Z', nap_z))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('bodem_NAP_Z_date', metadataDatum))

            df_alt1 = df[(df.Item.str.match("^ALT", flags=re.IGNORECASE))]
            df_alt1[df_alt1['NAP Z'].notnull()]
            df_alt1['metadataDatum'] = df_alt1['Datum']
            if not df_alt1.empty:
                df_alt = pd.merge_asof(left=df_datum, right=df_alt1[['Datum', 'NAP Z','SN', 'metadataDatum']], on='Datum', direction='backward')
                df_alt.fillna(df_alt1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_alt.iloc[0]['NAP Z']
                sn = int(df_alt.iloc[0]['SN'])
                metadataDatum = df_alt.iloc[0]['metadataDatum'].strftime("%Y-%m-%d")
                self.insert_tuple(request_location, 'ALT_NAP_Z', nap_z)
                self.insert_tuple(request_location, 'ALT_SerialNumber', sn)
                self.insert_tuple(request_location, 'ALT_NAP_Z_date', metadataDatum)
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ALT_NAP_Z', nap_z))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ALT_SerialNumber', sn))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ALT_NAP_Z_date', metadataDatum))

            df_stb1 = df[(df.Item.str.match("^STB", flags=re.IGNORECASE))]
            df_stb1[df_stb1['NAP Z'].notnull()]
            df_stb1['metadataDatum'] = df_stb1['Datum']
            if not df_stb1.empty:
                df_stb = pd.merge_asof(left=df_datum, right=df_stb1[['Datum', 'NAP Z','SN', 'metadataDatum']], on='Datum', direction='backward')
                df_stb.fillna(df_stb1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_stb.iloc[0]['NAP Z']
                sn = int(df_stb.iloc[0]['SN'])
                metadataDatum = df_stb.iloc[0]['metadataDatum'].strftime("%Y-%m-%d")
                self.insert_tuple(request_location, 'STB_NAP_Z', nap_z)
                self.insert_tuple(request_location, 'STB_SerialNumber', sn)
                self.insert_tuple(request_location, 'STB_NAP_Z_date', metadataDatum)
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('STB_NAP_Z', nap_z))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('STB_SerialNumber', sn))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('STB_NAP_Z_date', metadataDatum))

            df_adc1 = df[(df.Item.str.match("^ADC", flags=re.IGNORECASE))]
            df_adc1[df_adc1['NAP Z'].notnull()]
            df_adc1['metadataDatum'] = df_adc1['Datum']
            if not df_adc1.empty:
                df_adc = pd.merge_asof(left=df_datum, right=df_adc1[['Datum', 'NAP Z','SN', 'metadataDatum']], on='Datum', direction='backward')
                df_adc.fillna(df_adc1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_adc.iloc[0]['NAP Z']
                sn = int(df_adc.iloc[0]['SN'])
                metadataDatum = df_adc.iloc[0]['metadataDatum'].strftime("%Y-%m-%d")
                self.insert_tuple(request_location, 'ADC_NAP_Z', nap_z)
                self.insert_tuple(request_location, 'ADC_SerialNumber', sn)
                self.insert_tuple(request_location, 'ADC_NAP_Z_date', metadataDatum)
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADC_NAP_Z', nap_z))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADC_SerialNumber', sn))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADC_NAP_Z_date', metadataDatum))

            df_adv1 = df[(df.Item.str.match("^ADV", flags=re.IGNORECASE))]
            df_adv1[df_adv1['NAP Z'].notnull()]
            df_adv1['metadataDatum'] = df_adv1['Datum']
            if not df_adv1.empty:
                df_adv = pd.merge_asof(left=df_datum, right=df_adv1[['Datum', 'NAP Z','SN','Druk offset','metadataDatum']], on='Datum', direction='backward')
                df_adv.fillna(df_adv1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_adv.iloc[0]['NAP Z']
                sn = int(df_adv.iloc[0]['SN'])
                drukoffset = df_adv.iloc[0]['Druk offset']
                metadataDatum = df_adv.iloc[0]['metadataDatum'].strftime("%Y-%m-%d")
                self.insert_tuple(request_location, 'ADV_NAP_Z', nap_z)
                self.insert_tuple(request_location, 'ADV_druk_offset', drukoffset)
                self.insert_tuple(request_location, 'ADV_SerialNumber', sn)
                self.insert_tuple(request_location, 'ADV_NAP_Z_date', metadataDatum)
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADV_NAP_Z', nap_z))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADV_druk_offset', drukoffset))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADV_SerialNumber', sn))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADV_NAP_Z_date', metadataDatum))

            df_mpp1 = df[(df.Item.str.match("^MPP", flags=re.IGNORECASE))]
            df_mpp1[df_mpp1['NAP Z'].notnull()]
            df_mpp1['metadataDatum'] = df_mpp1['Datum']
            if not df_mpp1.empty:
                df_mpp = pd.merge_asof(left=df_datum, right=df_mpp1[['Datum', 'NAP Z','SN', 'metadataDatum']], on='Datum', direction='backward')
                df_mpp.fillna(df_mpp1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_mpp.iloc[0]['NAP Z']
                sn = int(df_mpp.iloc[0]['SN'])
                metadataDatum = df_mpp.iloc[0]['metadataDatum'].strftime("%Y-%m-%d")
                self.insert_tuple(request_location, 'MPP_NAP_Z', nap_z)
                self.insert_tuple(request_location, 'MPP_SerialNumber', sn)
                self.insert_tuple(request_location, 'MPP_NAP_Z_date', metadataDatum)
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('MPP_NAP_Z', nap_z))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('MPP_SerialNumber', sn))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('MPP_NAP_Z_date', metadataDatum))

            df_aqd1 = df[(df.Item.str.match("^AQD", flags=re.IGNORECASE))]
            df_aqd1[df_aqd1['NAP Z'].notnull()]
            df_aqd1['metadataDatum'] = df_aqd1['Datum']
            if not df_aqd1.empty:
                df_aqd = pd.merge_asof(left=df_datum, right=df_aqd1[['Datum', 'NAP Z','SN','Druk offset', 'metadataDatum']], on='Datum', direction='backward')
                df_aqd.fillna(df_aqd1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_aqd.iloc[0]['NAP Z']
                sn = int(df_aqd.iloc[0]['SN'])
                drukoffset = df_aqd.iloc[0]['Druk offset']
                metadataDatum = df_aqd.iloc[0]['metadataDatum'].strftime("%Y-%m-%d")
                self.insert_tuple(request_location, 'AQD_NAP_Z', nap_z)
                self.insert_tuple(request_location, 'AQD_druk_offset', drukoffset)
                self.insert_tuple(request_location, 'AQD_SerialNumber', sn)
                self.insert_tuple(request_location, 'AQD_NAP_Z_date', metadataDatum)
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('AQD_NAP_Z', nap_z))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('AQD_druk_offset', drukoffset))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('AQD_SerialNumber', sn))
                print("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('AQD_NAP_Z_date', metadataDatum))
                
        except XLRDError:
            print("ReadMetaDataAdjusted.parse_file: could not find sheet {} in {}".format(request_location, self.fn))
            raise

    def get_meta_data(self):
        return self.current_meta_data
